@@ -39,10 +39,11 @@ List nonlocal(arma::mat Y, arma::mat X, arma::vec prior_mean, arma::uvec penaliz
             Y = Y / sdy;
         }
         X = arma::join_rows(arma::ones<mat>(n, 1), X);
-        block_vec = join_cols(ones<vec>(1), block_vec);
+        block_vec = join_cols(ones<vec>(1), block_vec); // put intercept in the first block
         p = p + 1;
         N_blocks = N_blocks + 1;
         sdx = join_rows(ones<mat>(1,1), sdx);
+        penalize = arma::join_cols(arma::zeros<uvec>(1), penalize); // add one indicator of penalization for intercept. Do not penalize intercept    
     }else{
         if(standardize == true){
             Y = scaling(Y);
@@ -55,7 +56,7 @@ List nonlocal(arma::mat Y, arma::mat X, arma::vec prior_mean, arma::uvec penaliz
     arma::mat YY = trans(Y) * Y;
     arma::mat YX = trans(Y) * X;
     arma::mat XX = trans(X) * X;
-    penalize = find(penalize > 0);
+    arma::uvec penalize_index = find(penalize > 0);
 
     /*
     the input of penalize is (0,1,1,0,1...)
@@ -175,7 +176,7 @@ List nonlocal(arma::mat Y, arma::mat X, arma::vec prior_mean, arma::uvec penaliz
             nu = s * nu;
 
             // acceptance threshold
-            priorcomp = log_nonlocal_prior(b.rows(block_indexes(i), block_indexes(i+1)-1), vglobal, prior_mean.rows(block_indexes(i), block_indexes(i+1)-1)) - log_normal_density_matrix(b.rows(block_indexes(i), block_indexes(i+1)-1), eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular);
+            priorcomp = log_nonlocal_prior(b.rows(block_indexes(i), block_indexes(i+1)-1), vglobal, penalize.rows(block_indexes(i), block_indexes(i+1)-1), prior_mean.rows(block_indexes(i), block_indexes(i+1)-1)) - log_normal_density_matrix(b.rows(block_indexes(i), block_indexes(i+1)-1), eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular);
 
             u = arma::as_scalar(randu(1));
 
@@ -198,7 +199,7 @@ List nonlocal(arma::mat Y, arma::mat X, arma::vec prior_mean, arma::uvec penaliz
                 b.subvec(block_indexes(i), block_indexes(i+1) - 1) = betaprop + beta_hat_block;
 
             }else{
-                while (log_nonlocal_prior(beta_hat_block + betaprop, vglobal, prior_mean.rows(block_indexes(i), block_indexes(i+1)-1)) - log_normal_density_matrix(beta_hat_block + betaprop, eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular)< ly){
+                while (log_nonlocal_prior(beta_hat_block + betaprop, vglobal, penalize.rows(block_indexes(i), block_indexes(i+1)-1), prior_mean.rows(block_indexes(i), block_indexes(i+1)-1)) - log_normal_density_matrix(beta_hat_block + betaprop, eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular)< ly){
                     
                     loopcount += 1;
 
@@ -225,14 +226,8 @@ List nonlocal(arma::mat Y, arma::mat X, arma::vec prior_mean, arma::uvec penaliz
 
         vgprop = exp(log(vglobal) + arma::as_scalar(randn(1)) * 0.05);
 
-        // ratio = exp(betaprior(b.elem(penalize), vgprop * v.elem(penalize), prior_type, user_prior_function) + R::dnorm(vgprop, 0.5, 0.25, 1)  - betaprior(b.elem(penalize), vglobal  * v.elem(penalize), prior_type, user_prior_function) - R::dnorm(vglobal, 0.5, 0.25, 1)  +log(vgprop) - log(vglobal));
-        if(icept == false){
-            // if there is no intercept, pass the full vector
-            ratio = exp(log_nonlocal_prior(b.elem(penalize), vgprop, prior_mean) + log_normal_density(vgprop, 0.5, 100)  - log_nonlocal_prior(b.elem(penalize), vglobal, prior_mean) - log_normal_density(vglobal, 0.5, 100)  +log(vgprop) - log(vglobal));
-        }else{
-            // else, do not consider the intercept when evaluating prior function
-            ratio = exp(log_nonlocal_prior(b.rows(1,p-1), vgprop, prior_mean) + log_normal_density(vgprop, 0.5, 100)  - log_nonlocal_prior(b.rows(1,p-1), vglobal, prior_mean) - log_normal_density(vglobal, 0.5, 100)  +log(vgprop) - log(vglobal));
-        }
+        ratio = exp(log_nonlocal_prior(b, vgprop, penalize, prior_mean) + log_normal_density(vgprop, 0.5, 100)  - log_nonlocal_prior(b, vglobal, penalize, prior_mean) - log_normal_density(vglobal, 0.5, 100) + log(vgprop) - log(vglobal));
+
 
         if(as_scalar(randu(1)) < ratio){
             vglobal = vgprop;
