@@ -40,10 +40,11 @@ List ridge(arma::mat Y, arma::mat X, arma::uvec penalize, arma::vec block_vec, i
             Y = Y / sdy;
         }
         X = arma::join_rows(arma::ones<mat>(n, 1), X);
-        block_vec = join_cols(ones<vec>(1), block_vec);
+        block_vec = join_cols(ones<vec>(1), block_vec); // put intercept in the first block
         p = p + 1;
         N_blocks = N_blocks + 1;
         sdx = join_rows(ones<mat>(1,1), sdx);
+        penalize = arma::join_cols(arma::zeros<uvec>(1), penalize); // add one indicator of penalization for intercept. Do not penalize intercept    
     }else{
         if(standardize == true){
             Y = scaling(Y);
@@ -56,7 +57,7 @@ List ridge(arma::mat Y, arma::mat X, arma::uvec penalize, arma::vec block_vec, i
     arma::mat YY = trans(Y) * Y;
     arma::mat YX = trans(Y) * X;
     arma::mat XX = trans(X) * X;
-    penalize = find(penalize > 0);
+    arma::uvec penalize_index = find(penalize > 0);
 
     /*
     the input of penalize is (0,1,1,0,1...)
@@ -182,7 +183,7 @@ List ridge(arma::mat Y, arma::mat X, arma::uvec penalize, arma::vec block_vec, i
             nu = s * nu;
 
             // acceptance threshold
-            priorcomp = log_ridge_prior(b.rows(block_indexes(i), block_indexes(i+1)-1) , lambda, vglobal) - log_normal_density_matrix(b.rows(block_indexes(i), block_indexes(i+1)-1), eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular);
+            priorcomp = log_ridge_prior(b.rows(block_indexes(i), block_indexes(i+1)-1) , lambda, vglobal, penalize.subvec(block_indexes(i), block_indexes(i+1) - 1)) - log_normal_density_matrix(b.rows(block_indexes(i), block_indexes(i+1)-1), eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular);
 
             u = arma::as_scalar(randu(1));
 
@@ -204,7 +205,7 @@ List ridge(arma::mat Y, arma::mat X, arma::uvec penalize, arma::vec block_vec, i
                 b.subvec(block_indexes(i), block_indexes(i+1) - 1) = betaprop + beta_hat_block;
 
             }else{
-                while (log_ridge_prior(beta_hat_block + betaprop, lambda, vglobal) - log_normal_density_matrix(beta_hat_block + betaprop, eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular) < ly){
+                while (log_ridge_prior(beta_hat_block + betaprop, lambda, vglobal, penalize.subvec(block_indexes(i), block_indexes(i+1) - 1)) - log_normal_density_matrix(beta_hat_block + betaprop, eye<mat>(block_vec(i), block_vec(i)) / pow(s,2) / eta, singular) < ly){
                     
                     loopcount += 1;
 
@@ -230,11 +231,9 @@ List ridge(arma::mat Y, arma::mat X, arma::uvec penalize, arma::vec block_vec, i
 
         // update lambda
         lambda_prop = exp(log(lambda) + arma::as_scalar(randn(1)) * 0.05 );
-        if(icept == false){
-            ratio = exp(log_ridge_prior(b.elem(penalize), lambda_prop, vglobal) + log_normal_density(lambda_prop, 0.0, 100.0)  - log_ridge_prior(b.elem(penalize), lambda, vglobal) - log_normal_density(lambda, 0.0, 100.0)  +log(lambda_prop) - log(lambda));
-        }else{
-            ratio = exp(log_ridge_prior(b.rows(1,p-1), lambda_prop, vglobal) + log_normal_density(lambda_prop, 0.0, 100.0)  - log_ridge_prior(b.rows(1,p-1), lambda, vglobal) - log_normal_density(lambda, 0.0, 100.0)  +log(lambda_prop) - log(lambda));
-        }
+
+        ratio = exp(log_ridge_prior(b, lambda_prop, vglobal, penalize) + log_normal_density(lambda_prop, 0.0, 100.0)  - log_ridge_prior(b, lambda, vglobal, penalize) - log_normal_density(lambda, 0.0, 100.0)  + log(lambda_prop) - log(lambda));
+
         if(as_scalar(randu(1)) < ratio){
             lambda = lambda_prop;
         }
@@ -242,11 +241,9 @@ List ridge(arma::mat Y, arma::mat X, arma::uvec penalize, arma::vec block_vec, i
 
         // update the global shrinkage parameter
         vgprop = exp(log(vglobal) + arma::as_scalar(randn(1)) * 0.05);
-        if(icept == false){
-            ratio = exp(log_ridge_prior(b.elem(penalize), lambda, vgprop) + log_normal_density(vgprop, 0.0, 100.0)  - log_ridge_prior(b.elem(penalize), lambda, vglobal) - log_normal_density(vglobal, 0.0, 100.0)  +log(vgprop) - log(vglobal));
-        }else{
-            ratio = exp(log_ridge_prior(b.rows(1,p-1), lambda, vgprop) + log_normal_density(vgprop, 0.0, 100.0)  - log_ridge_prior(b.rows(1,p-1), lambda, vglobal) - log_normal_density(vglobal, 0.0, 100.0)  +log(vgprop) - log(vglobal));
-        }
+
+        ratio = exp(log_ridge_prior(b, lambda, vgprop, penalize) + log_normal_density(vgprop, 0.0, 100.0)  - log_ridge_prior(b, lambda, vglobal, penalize) - log_normal_density(vglobal, 0.0, 100.0)  +log(vgprop) - log(vglobal));
+
         if(as_scalar(randu(1)) < ratio){
             vglobal = vgprop;
         }
